from decimal import Decimal

from django.contrib.humanize.templatetags.humanize import intcomma


def humanize_amount(amount_value):
    """
    This function is used to format the provided amount value into a human readable format.

    :param amount_value: float: Amount value to be formatted
    :return: str: Comma separated amount value
    """
    if amount_value:
        return intcomma(amount_value)
    return 0


def to_decimal(x):
    """
    This function is used to convert a numeric value into a decimal value.

    :param x:
    :return: Decimal:
    """
    if not x:
        x = 0
    return Decimal(x)


def to_int(x):
    """
    This function is used to convert a numeric value into an int value.

    :param x:
    :return: int:
    """
    if not x:
        x = 0
    return int(x)


def is_greater(a, b):
    """
    This function takes in two numeric values and compares if one is greater than the other.

    :param a:
    :param b:
    :return: bool: Returns True or False
    """
    return to_decimal(a) > to_decimal(b)


def is_less(a, b):
    """
    This function takes in two numeric values and compares if one is less than the other.

    :param a:
    :param b:
    :return: bool: Returns True or False
    """
    return to_decimal(a) < to_decimal(b)


def is_equal(a, b):
    """
    This function takes in two numeric values and compares if the two values are equal.

    :param a:
    :param b:
    :return: bool: Returns True or False
    """
    return to_decimal(a) == to_decimal(b)


def is_greater_or_equal(a, b):
    """
    This function takes in two numeric values and compares if one is greater or equal to the other.

    :param a:
    :param b:
    :return: bool: Returns True or False
    """
    return to_decimal(a) >= to_decimal(b)


def add(a, b):
    """
    This function takes in two numeric values and returns a sum of those two numbers.

    :param a:
    :param b:
    :return: Decimal:
    """
    return to_decimal(a) + to_decimal(b)


def multiply(a, b):
    """
    This function takes in two numeric values and multiplies the two numbers.

    :param a:
    :param b:
    :return: Decimal:
    """
    return to_decimal(a) * to_decimal(b)


def divide(a, b):
    """
    This function takes in two numeric values and divides the two numbers.

    :param a:
    :param b:
    :return: Decimal:
    """
    return to_decimal(a) / to_decimal(b)


def subtract(a, b):
    """
    This function takes in two numeric values and subtracts one from the other.

    :param a:
    :param b:
    :return: Decimal:
    """
    return to_decimal(a) - to_decimal(b)



class GenericException(Exception):
    """
    This is the base class from which all system exceptions inherit
    """

    def __init__(self, message=None, error_code=None, generic_error=None, exception_object=None, *args, **kwargs):
        """
        This function is used to initialize an exception object.


        :param error_code: str: Represents an error code used to uniquely identify your system errors
        :param message: str: Represents an error message that you want to display to the user
        :param generic_error: str: Represents the actual error message that occurred, that you may want to store/log
        :param exception_object: Exception: Represents an exception object
        :param args: args: Any arguments
        :param kwargs: kwargs: Any keyword arguments
        """
        super(GenericException, self).__init__(self, error_code, message, *args, **kwargs)
        self.message = message
        self.generic_error = generic_error
        self.error_code = error_code
        self.exception_object = exception_object

    def __str__(self):
        """
        This function is used to return the exception message when an object is called.

        :return: str: Returns an error message
        """
        return self.message


class InvalidCredentialsException(GenericException):
    """
    This class is used to define an exception type raised when a user supplies
    invalid credentials when attempting to login.
    """


class InvalidOrganizationException(GenericException):
    """
    This class is used to define an exception type raised when the application fails to resolve
    the user's organization derived from the provided user name when attempting to login.
    """


class NullObjectException(GenericException):
    """
    This class is used to define an exception type raised when the application detects
    a missing object that is expected to contain a value.
    """


class ValidationFailedException(GenericException):
    """
   This class is used to define an exception type raised when provided data does not meet
   the validation rules required by a given operation.
    """


class OperationFailedException(GenericException):
    """
   This class is used to define an exception type raised when a given operation
   is violated.
    """



import enum


class RecordStatus(enum.Enum):
    ACTIVE = "Active"
    DELETED = "Delete"
    ACTIVE_LOCKED = "Active Locked"

    def describe(self):
        """
        This function is used to get detailed information about an enum object.

        For example;
            when RecordStatus.ACTIVE.describe() is called, it's name and value will be returned i.e. ACTIVE, Active
        :return:
        """
        return self.name, self.value

    def __str__(self):
        """
        This function is used to get the object name when an object is called.

        :return: str
        """
        return self.name


class Gender(enum.Enum):
    MALE = "Male"
    FEMALE = "Female"

    def describe(self):
        """
        This function is used to get detailed information about an enum object.

        For example;
            when Gender.MALE.describe() is called, it's name and value will be returned i.e. MALE, Male
        :return:
        """
        return self.name, self.value

    def __str__(self):
        """
        This function is used to get the object name when an object is called.

        :return: str
        """
        return self.name


class FileAttribute(enum.Enum):
    """
    This class contains operations that can be done on a file.

    For example;
        1. Create a new file
        2. Read from an existing file
        3. Append data to an existing file
    """
    CREATE = ('Write to file', 'w+')
    READ = ('Read from file', 'r+')
    APPEND = ('Append data to a file', 'a+')

    def __init__(self, display_name, code_name):
        self.display_name = display_name  # Display name
        self.code_name = code_name  # Attribute of a file object tells you which mode a file was opened in

    def describe(self):
        """
        This function is used to get detailed information about an enum object.

        For example;
            when FileAttribute.CREATE.describe() is called, it's name and value will be returned i.e. CREATE, Write to file
        :return:
        """
        return self.name, self.display_name

    def __str__(self):
        """
        This function is used to get the object name when an object is called.

        :return: str
        """
        return self.name


class SeverityLevel(enum.Enum):
    """
    This class represents the severity of the log message that is being logged to the logging microservice
    """
    DEBUG = "Debug"
    INFO = "Info"
    CRITICAL = "Critical"
    FATAL = "Fatal"
    ERROR = "Error"
    WARNING = "Warning"

    def describe(self):
        """
        This function is used to get detailed information about an enum object.

        For example;
            when SeverityLevel.DEBUG.describe() is called, it's name and value will be returned i.e. DEBUG, Debug
        :return: tuple
        """
        return self.name, self.value

    def __str__(self):
        """
        This function is used to get the object name when an object is called.

        :return: str
        """
        return self.name


class LogStatus(enum.Enum):
    """
    This class represents the validity of data that is being sent to the logging microservice
    """
    VALID = "Valid"
    INVALID = "Invalid"

    def describe(self):
        """
        This function is used to get detailed information about an enum object.

        For example;
            when LogStatus.VALID.describe() is called, it's name and value will be returned i.e. VALID, Valid
        :return:
        """
        return self.name, self.value

    def __str__(self):
        """
        This function is used to get the object name when an object is called.

        :return: str
        """
        return self.name



import decimal
import json
import logging
import re
import sys
import uuid


def default_search_form_input_styles(width: float = 25):
    """
    This function is used to compose the default input field css styles that are applied on the search input.

    :param width: float: Maximum size of the input fields
    :return: dict: Returns a dictionary containing input field css styles
    """
    return {"placeholder": 'ENTER SEARCH TERM', "class": "form-control input-widget-size",
            'style': f'width:{width}%; margin-left:0.5em; margin-right:0.2em; height: calc(2.2rem + 2px); margin-top: 0.3em;'}


def generate_uuid():
    """
    This function is used to generate a uuid string.

    :return: str: uuid string
    """
    return uuid.uuid4().hex


def default_input_widget_styles(place_holder):
    """
    These are the default css classes used when creating input field user interface forms.

    :param place_holder: str: Input field placeholder
    :return: dict: Returns a dictionary containing field's css styles
    """
    return {"placeholder": place_holder, "class": "form-control input-widget-size"}


def default_datetime_widget_styles(place_holder):
    """
    This function contains the default css classes used when creating input field user interface forms.

    :param place_holder: str: Input field placeholder
    :return: dict: Returns a dictionary containing field's css styles
    """
    return {"placeholder": place_holder, "class": "form-control datetimepicker input-widget-size"}


def default_date_widget_styles(place_holder):
    """
    This function contains the default css classes used when creating a date field user interface forms.

    :param place_holder: str: Input field placeholder
    :return: dict: Returns a dictionary containing field's css styles
    """
    return {"placeholder": place_holder, "class": "form-control datepicker input-widget-size"}


def default_drop_down_widget_styles(place_holder):
    """
    This function contains the default css classes used when creating a drop down select field user interface forms.

    :param place_holder: str: Input field placeholder
    :return: dict: Returns a dictionary containing field's css styles
    """
    return {'data-size': 7, 'title': place_holder, "class": "selectpicker"}


def default_checkbox_input_widget_styles():
    """
    This function contains the default css classes used when creating a checkbox input field user interface forms.

    :return: dict: Returns a dictionary containing field's css styles
    """
    return {"class": "form-check-input"}


def default_multiple_checkbox_input_widget_styles():
    """
    This function contains the default css classes used when creating a checkbox input field user interface forms.

    :return: dict: Returns a dictionary containing field's css styles
    """
    return {"class": "checkbox-style"}


def default_bootstrap_checkbox_input_widget_styles():
    """
    This function contains the default css classes used when creating a checkbox input field user interface forms.

    :return: dict: Returns a dictionary containing field's css styles
    """
    return {"class": "bootstrap-switch",
            "data-on-label": "<i class='tim-icons icon-check-2'></i>",
            "data-off-label": "<i class='tim-icons icon-simple-remove'></i>"}


def default_text_area_widget_styles(place_holder: str, size_in_pixels: int = 300):
    """
    This function contains the default css classes used when creating a text area user interface forms.

    :param place_holder: str: Input field placeholder
    :param size_in_pixels: int: Represents the text-area vertical height in pixels
    :return: dict: Returns a dictionary containing field's css styles
    """
    return {"placeholder": place_holder, "class": "form-control",
            'style': f'max-height: {size_in_pixels}px; border: 1px solid rgba(34, 42, 66, 0.7);'}


def extract_numeric_values(input_value):
    """
    This function is used to extract numeric values from the provided input and put in an array.

    :param input_value:
    :return:
    """
    if not input_value:
        return []
    input_value = str(input_value)
    return [int(s) for s in re.findall(r'\b\d+\b', input_value)]


def set_paginated_data_to_content(request, context: dict, paginator, show_paginator_summary: bool = True,
                                  record_per_page: int = 100):
    """
   This function is used to set paginated data together with the search form to the request context

    :param context: dict: Dictionary holding request details
    :param paginator: Paginator: Django paginator object
    :param request: request: HTTP request object
    :param show_paginator_summary: bool: Boolean flag used on the UI to show/hide the paginator summary
    :param record_per_page: int: Maximum number records per query set
    :return:
    """
    current_page = None
    if 'page' in request.GET:
        current_page = int(request.GET["page"])
    if current_page is None:
        current_page = 1
    page_object = paginator.get_page(current_page)
    page = paginator.page(current_page)
    start_index = page.start_index()
    end_index = page.end_index()
    if current_page != paginator.num_pages:
        end_index = (int(start_index) + record_per_page) - 1  # Subtract 1 because of the last index overflow

    context['records'] = page_object
    context['paginator'] = paginator
    context['current_page'] = current_page
    if show_paginator_summary:
        context['paginator_label'] = f'Showing {start_index} to {end_index} of {paginator.count} entries'


def add_search_term(search_term: str):
    """
    This function is used to construct a search by like query string that  is used by SQLAlchemy.

    :param search_term: str: Represents a search term to which the like operator is to be applied
    :return:
    """
    return '%{}%'.format(search_term)


def remove_all_white_space_and_new_line(data: str):
    """
    This function is used to remove all whitespaces, tabs and new lines from the received data string.

    :param data: str: Represents a string that this operation is to be applied on.
    :return: str: Returns a string
    """
    return re.sub(r"[\n\t\s]*", "", data)


def clean_multiple_spaces(value: str):
    """
    This function is used to remove all multiple white spaces and replace them with a single space

    :param value: str: Represents a data source that is to be cleaned
    :return: str: Returns a string without multiple spaces
    """
    return " ".join(value.split())


def remove_multiple_spaces(value: str):
    """
    This function is used to remove multiple white spaces from the search term
    and split the resultant value by a single space

    :param value: str: Represents a string to be cleaned up
    :return: str: Returns a string without multiple spaces
    """
    return clean_multiple_spaces(value).split(' ')


def is_place_holder_exists(place_holder: str, text: str):
    """
    This function is used to check if a place holder exists in the provided template

    :param place_holder:
    :param text:
    :return:
    """
    clean_data = clean_multiple_spaces(text)
    patterns = '(?i)%\\s*{}\\s*%'.format(place_holder)
    if re.search(patterns, clean_data):
        return True
    return False


def replace_placeholders(data_source: str, key_names_list: list, values_dictionary: dict):
    """
    This function is used to replace placeholders within % placeholder % with a value from the replacements.

    Note:
        replacement values should be placed in the values_dictionary
        for example if key_names_list = ['username','password']
        then values_dictionary = {'username':'some username', 'password':'some password'}

    :param data_source: str: Represents a data set on which value replacement is to be performed.
    :param key_names_list: list: Represents a list of keys expected to be present in the datasource
    :param values_dictionary: dict: Represents a key to value pair of dats that is to be replaced with placeholders
    :return: str: Returns a string with all specified placeholders replaced with actual values
    """
    try:
        if not data_source:
            return data_source
        pattern = r'\%(.+?)\%'
        placeholders = re.findall(pattern, data_source)

        for n, i in enumerate(placeholders):
            for key in key_names_list:
                if i == key and key in values_dictionary:
                    placeholders[n] = values_dictionary[key]

        items = iter(str(el) for el in placeholders)
        return re.sub(pattern, lambda L: next(items), data_source)
    except Exception as ex:
        logging.error(msg=ex)
        pass


def get_image_full_path(request, image):
    """
    This function is used to obtain an image url using the provided image object.

    :param request: request: HTTP request object
    :param image: ImageField: Represents the image from which an image url can be obtained
    :return:
    """
    if image and request:
        domain_root = request._current_scheme_host
        image_url = image.url
        return str(domain_root) + str(image_url)
    return None


class DecimalEncoder(json.JSONEncoder):
    """
    This class is used to encode decimal values into acceptable python json objects that are to be sent through an API.
    """

    def default(self, o):
        if isinstance(o, decimal.Decimal):
            return float(o)
        return super(DecimalEncoder, self).default(o)


def get_processed_json_object(data: str):
    """
    Serialize data into an acceptable python json object

    :param data: str: Represents a string to be converted to json
    :return: json: Returns a json object
    """
    return json.dumps(data, cls=DecimalEncoder)


def get_class_and_method_name():
    """
    This function is used to get the class and method name from the context in which this function is called.

    :return:
    """
    frame = sys._getframe(1)
    try:
        class_name = frame.f_locals['self'].__class__.__name__
    except KeyError:
        class_name = None
    method_name = frame.f_code.co_name
    return class_name, method_name


def get_client_ip_address(request):
    """
    This function is used to get the client's ip address from an HTTP request.

    :param request:
    :return:
    """
    # The X-Forwarded-For (XFF) HTTP header field is a common method for identifying the originating IP address
    # of a client connecting to a web server through an HTTP proxy or load balancer.
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip_address = x_forwarded_for.split(',')[-1].strip()
    else:
        # Client's ip address
        ip_address = request.META.get('REMOTE_ADDR', None)
    return ip_address


def get_util_logger(file_name):
    filename = f'_{file_name}.log'
    open(filename, 'a+')
    logging.basicConfig(filename=filename, level=logging.DEBUG,
                        format='[%(asctime)s] %(levelname)s [%(funcName)s:%(lineno)d] %(message)s')
    return logging.getLogger(f'{file_name}')



import calendar
import datetime
from datetime import timedelta, date

from django.contrib.humanize.templatetags.humanize import naturalday

ISO_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
DATE_FORMAT = '%m/%d/%Y'
DATE_TIME_FORMAT = '%m/%d/%Y %I:%M %p'
DATE_TIME_WITH_SECONDS_FORMAT = '%m/%d/%Y %I:%M:%S %p'
REPORT_DATE_TIME_FORMAT = '%m%d%Y%I%M%p'


def now():
    """
    This function is used to get the current date and time object.

    :return: datetime:
    """
    return datetime.datetime.now()


def format_date(data_value):
    """
    This function is used to format the provided date object into a human readable format.

    :param data_value: date or datetime: Date object to be formatted
    :return: str: Human readable date format
    """

    if data_value:
        return naturalday(data_value)
    return None


def is_date_less_than_now(data_value):
    """
    This function is used to compare if the provided date object is less than the current date.
    This returns True if the provided date is less than the current date and False otherwise

    :param data_value: date: The date object to be compared to now
    :return: bool:
    """
    if to_start_of_day(data_value) < to_start_of_day(now()):
        return True
    return False


def get_current_year():
    """
    This function is used to get the current year integer representation (yyyy).

    :return: int: Current year
    """
    current_date = datetime.datetime.now()
    return current_date.year


def get_short_year():
    """
    This function is used to get the current Year, short version, without the century.

    :return: int: Return has two digits of the current year
    """
    current_date = datetime.datetime.now()
    return current_date.strftime("%y")


def current_time_milliseconds():
    """
    This function is used to get the current time in milliseconds.

    :return: long: Returns time in milliseconds
    """
    import time
    return round(time.time() * 1000)


def subtract_days(date_object, no_of_days):
    """
    Takes back the provided date a given number of days

    :param date_object: date: Date object to subject days from
    :param no_of_days: int: Number of days to be removed
    :return: date:
    """
    return date_object - timedelta(days=float(no_of_days))


def add_days(date_object, no_of_days):
    """
    Adds number of days to the provided date

    :param date_object: date: Date object to add days to
    :param no_of_days: int: Number of days to be added
    :return: date:
    """
    return date_object + timedelta(days=float(no_of_days))


def get_hours_between_dates(lower_date, higher_date):
    """
    Returns number of hours between two dates

    :param lower_date:
    :param higher_date:
    :return:
    """
    lower_date = lower_date.replace(tzinfo=None)
    higher_date = higher_date.replace(tzinfo=None)

    diff = higher_date - lower_date
    days, seconds = diff.days, diff.seconds
    hours = days * 24 + seconds // 3600
    minutes = (seconds % 3600) // 60
    seconds = seconds % 60
    return hours + (minutes / 60) + (seconds / (60 * 60))


def get_days_between_dates(lower_date, higher_date):
    """
    Returns number of days between two dates

    :param lower_date:
    :param higher_date:
    :return:
    """
    lower_date = lower_date.replace(tzinfo=None)
    higher_date = higher_date.replace(tzinfo=None)

    diff = higher_date - lower_date
    return diff.days


def get_datetime_from_string(date_value, date_format=DATE_TIME_FORMAT):
    """
    This function is used to get a python date object using the provided date format.

    :param date_value:
    :param date_format:
    :return:
    """
    return datetime.datetime.strptime(date_value, date_format)


def get_html_datetime(date_object, date_format=DATE_TIME_FORMAT):
    """
    This function is used to format a date object into a format specified in the provided date format.

    :param date_object:
    :param date_format:
    :return:
    """
    if date_object:
        return date_object.strftime(date_format)
    return now().strftime(date_format)


def get_report_name_extension():
    return now().strftime(REPORT_DATE_TIME_FORMAT)


def get_week_days():
    """
    This function is used to get  a list of calendar days starting from Monday to Sunday.

    :return: list:
    """
    return list(calendar.day_name)


def get_current_week_day_number():
    """
    This function is used to get the numeric representation of the current day of the week.

    :return:
    """
    my_date = date.today()
    return my_date.weekday()


def get_billing_cycle_days():
    """
    This function is used to obtain the number of days remaining to reach the end of the week.

    :return: int: Returns the number of days
    """
    my_date = date.today()
    current_day = my_date.weekday()
    """ The maximum day index for python calendar for 0=monday , 6=sunday """
    max_days_index = 6
    return max_days_index - int(current_day)


def to_start_of_day(date_object):
    """
    This function is used to push time for the provided date object to the start of the day i.e. 00:00:00.

    :param date_object: date: Represents a date object
    :return: date: Return a date object with time pushed to the start of day
    """
    if not date_object:
        return None
    return datetime.datetime.combine(date_object, datetime.datetime.min.time())


def to_end_of_day(date_object):
    """
    This function is used to push time for the provided date object to the end of the day i.e. 11:59:59.

    :param date_object: date: Represents a date object
    :return: date: Return a date object with time pushed to the end of day
    """
    if not date_object:
        return None
    return datetime.datetime.combine(date_object, datetime.datetime.max.time())


def get_start_and_end_dayofweek(dayofweek):
    """
    This function is used to get a tuple containing the start date and end date of the week using the provided date object.

    For example;
        if a given date on wednesday is provided,
        the returned tuple will contain the start date and end date of the week the date lies in.

    :param dayofweek: date: Represents the date object on which you want to get it's start and end dates of it's week.
    :return: tuple: Containing the start and end date of the week
    """
    from datetime import datetime, timedelta
    dayofweek = dayofweek.strftime(DATE_FORMAT)
    date_object = datetime.strptime(dayofweek, DATE_FORMAT)
    start = date_object - timedelta(days=date_object.weekday())
    end = start + timedelta(days=6)
    return to_start_of_day(start), to_end_of_day(end)


def add_months(sourcedate, months):
    """
    This function is used to add a given number of months to the provided date object.

    :param sourcedate: date: The date object to which months are to be added
    :param months: int: Number of months to be added to the sourcedate
    :return: date: Date object in the future
    """
    month = sourcedate.month - 1 + int(months)
    year = sourcedate.year + month // 12
    month = month % 12 + 1
    day = min(sourcedate.day, calendar.monthrange(year, month)[1])
    return datetime.date(year, month, day)



from setuptools import setup, find_packages

classifiers = [
   'Development Status :: 5 - Production/Stable',
   'Intended Audience :: Developers',
   'Operating System :: Microsoft :: Windows :: Windows 10',
   'License :: OSI Approved :: MIT License',
   'Programming Language :: Python :: 3'
]

setup(
   name='paygo_utils',
   version='0.0.10',
   description='This is a utilities library that contains the base files needed by every microservice',
   long_description=open('README.txt').read() +'\n\n'+ open('CHANGELOG.txt').read(),
   url='',
   author='Umar Kayondo',
   author_email='umabravo70@gmail.com',
   license='MIT',
   classifiers=classifiers,
   keywords='paygo utils',
   packages=['paygo_utils'],
   install_requires=['']
)